@@ -28,6 +28,10 @@
 #include <gdk/gdk.h>
 #include <gdk/gdkkeysyms.h>
 
+#ifdef HAVE_BASENAME
+#include <libgen.h>
+#endif
+
 #ifdef ENABLE_XEMBED
 #include <gdk/gdkx.h>
 #endif /* ENABLE_XEMBED */
@@ -35,17 +39,63 @@
 #define DEFAULT_WIDTH  240
 #define DEFAULT_HEIGHT 240
 
+static char * _rsvg_basename(const char * file)
+{
+#ifdef HAVE_BASENAME
+	if(file && *file) {
+		char * file_dup = g_strdup(file);
+		return basename(file_dup);
+	}
+#endif
+
+	return NULL;
+}
+
 typedef struct _ViewerCbInfo ViewerCbInfo;
 struct _ViewerCbInfo
 {
 	GtkWidget  * window;
 	GtkWidget  * popup_menu;
+	GtkWidget  * image; /* the image widget */
+
 	GdkPixbuf  * pixbuf;
 	GByteArray * svg_bytes;
 	GtkAccelGroup * accel_group;
 	char * base_uri;
 };
 
+static void
+zoom_image(ViewerCbInfo * info, gint width, gint height)
+{
+	struct RsvgSizeCallbackData size_data;
+	GdkPixbuf * save_pixbuf = info->pixbuf;
+
+	size_data.type = RSVG_SIZE_WH;
+	size_data.width = width;
+	size_data.height = height;
+	size_data.keep_aspect_ratio = FALSE;
+
+	info->pixbuf = rsvg_pixbuf_from_data_with_size_data (info->svg_bytes->data, info->svg_bytes->len, &size_data, info->base_uri, NULL);
+	gtk_image_set_from_pixbuf(GTK_IMAGE(info->image), info->pixbuf);
+
+	if(save_pixbuf)
+		g_object_unref(G_OBJECT(save_pixbuf));
+}
+
+static void
+zoom_in(GObject * ignored, ViewerCbInfo * info)
+{
+	if(!info->pixbuf) return;
+	zoom_image(info, gdk_pixbuf_get_width(info->pixbuf) * 1.25, gdk_pixbuf_get_height(info->pixbuf) * 1.25);
+}
+
+static void
+zoom_out(GObject * ignored, ViewerCbInfo * info)
+{
+	if(!info->pixbuf) return;
+	zoom_image(info, gdk_pixbuf_get_width(info->pixbuf) / 1.25, gdk_pixbuf_get_height(info->pixbuf) / 1.25);
+}
+
 static void
 rsvg_window_set_default_icon (GtkWindow * window, GdkPixbuf *src)
 {
@@ -93,7 +143,7 @@ print_pixbuf (GObject * ignored, gpointer user_data)
 	GtkWidget *gpd;	
 	gint result;
 
-	gpd = gnome_print_dialog_new (gnome_print_job_new (gnome_print_config_default ()), _("Print SVG"), 0);
+	gpd = gnome_print_dialog_new (gnome_print_job_new (gnome_print_config_default ()), (unsigned char *)_("Print SVG"), 0);
 	gtk_window_set_transient_for(GTK_WINDOW (gpd), GTK_WINDOW (info->window));
 			  
 	if ((result = gtk_dialog_run (GTK_DIALOG (gpd))) != GNOME_PRINT_DIALOG_RESPONSE_CANCEL) 
@@ -148,7 +198,7 @@ print_pixbuf (GObject * ignored, gpointer user_data)
 			rowstride = gdk_pixbuf_get_rowstride (image);
 			pixels    = gdk_pixbuf_get_pixels (image);
 			
-			gnome_print_beginpage(gpc, "1");
+			gnome_print_beginpage(gpc, (unsigned char *)"1");
 			gnome_print_gsave (gpc);
 			gnome_print_translate (gpc, 0, page_height - height);
 			gnome_print_scale (gpc, width, height);
@@ -166,7 +216,7 @@ print_pixbuf (GObject * ignored, gpointer user_data)
 				{
 					GtkWidget * preview;
 					
-					preview = gnome_print_job_preview_new (gpm, _("SVG Print Preview"));
+					preview = gnome_print_job_preview_new (gpm, (unsigned char *)_("SVG Print Preview"));
 					gtk_window_set_transient_for(GTK_WINDOW(preview), GTK_WINDOW(info->window));
 					gtk_widget_show (GTK_WIDGET (preview));
 				}
@@ -183,11 +233,15 @@ print_pixbuf (GObject * ignored, gpointer user_data)
 #if GTK_CHECK_VERSION(2,4,0)
 
 static char *
-save_file (const char * title, GtkWidget * parent)
+save_file (const char * title, 
+		   const char * suggested_filename,
+		   GtkWidget * parent,
+		   int *success)
 {
 	GtkWidget *dialog;
 	char *filename = NULL;
 
+	*success = 0;
 	dialog = gtk_file_chooser_dialog_new (title,
 										  GTK_WINDOW (parent),
 										  GTK_FILE_CHOOSER_ACTION_SAVE,
@@ -195,10 +249,14 @@ save_file (const char * title, GtkWidget * parent)
 										  GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
 										  NULL);
 
+	if(suggested_filename && *suggested_filename) {
+		gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER (dialog), suggested_filename);
+	}
+
 	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
-		{
-			
+		{			
 			filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
+			*success = 1;
 		}
 	
 	gtk_widget_destroy (dialog);
@@ -209,17 +267,28 @@ save_file (const char * title, GtkWidget * parent)
 #else
 
 static char *
-save_file (const char * title, GtkWidget * parent)
+save_file (const char * title, 
+		   const char * suggested_filename,
+		   GtkWidget * parent,
+		   int *success)
 {
 	GtkWidget * filesel;
 	char * filename = NULL;
-			
+	
+	*success = 0;
 	filesel = gtk_file_selection_new (title);
 	gtk_window_set_transient_for(GTK_WINDOW(filesel), parent);
+
+	if(suggested_filename && *suggested_filename) {
+		char * utf8_suggestion = g_filename_from_utf8(suggested_filename);
+		gtk_file_selection_set_filename(GTK_FILE_SELECTION (filesel), utf8_suggestion);
+		g_free(utf8_suggestion);
+	}
 	
 	if (gtk_dialog_run (GTK_DIALOG (filesel)) == GTK_RESPONSE_OK)
 		{			
 			filename = g_strdup (gtk_file_selection_get_filename (GTK_FILE_SELECTION (filesel)));
+			*success = 1;
 		}
 
 	gtk_widget_destroy (dialog);
@@ -233,10 +302,19 @@ static void
 save_pixbuf (GObject * ignored, gpointer user_data)
 {
 	ViewerCbInfo * info = (ViewerCbInfo *)user_data;
-	char * filename;
+	char * filename, *base_name, *filename_suggestion;
+	int success = 0;
+
+	base_name = _rsvg_basename(info->base_uri);
+	if(base_name)
+		filename_suggestion = g_strdup_printf("%s.png", base_name);
+	else
+		filename_suggestion = NULL;
+
+	filename = save_file (_("Save SVG as PNG"), filename_suggestion, info->window, &success);
+	g_free(base_name);
+	g_free(filename_suggestion);
 
-	filename = save_file (_("Save SVG as PNG"), info->window);
-	
 	if (filename) 
 		{
 			GError * err = NULL;
@@ -262,7 +340,7 @@ save_pixbuf (GObject * ignored, gpointer user_data)
 
 			g_free (filename);
 		} 
-	else 
+	else if(success)
 		{
 			GtkWidget * errmsg;
 			
@@ -282,9 +360,12 @@ static void
 save_svg (GObject * ignored, gpointer user_data)
 {
 	ViewerCbInfo * info = (ViewerCbInfo *)user_data;
-	char * filename;
-	
-	filename = save_file (_("Save SVG"), info->window);
+	char * filename, *base_name;
+	int success = 0;
+
+	base_name = _rsvg_basename(info->base_uri);
+	filename = save_file (_("Save SVG"), base_name, info->window, &success);
+	g_free(base_name);
 
 	if (filename) 
 		{
@@ -310,7 +391,7 @@ save_svg (GObject * ignored, gpointer user_data)
 			else
 				{
 					size_t written = 0, remaining = info->svg_bytes->len;
-					const char * buffer = info->svg_bytes->data;
+					const unsigned char * buffer = info->svg_bytes->data;
 					
 					while (remaining > 0) {
 						written = fwrite (buffer + (info->svg_bytes->len - remaining), 1, 
@@ -341,7 +422,7 @@ save_svg (GObject * ignored, gpointer user_data)
 
 			g_free (filename);
 		} 
-	else 
+	else if(success)
 		{
 			GtkWidget * errmsg;
 					
@@ -418,6 +499,20 @@ create_popup_menu (ViewerCbInfo * info)
 	gtk_widget_add_accelerator(menu_item, "activate", info->accel_group, GDK_P, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
 #endif
 
+	menu_item = gtk_image_menu_item_new_from_stock (GTK_STOCK_ZOOM_IN, NULL);
+	g_signal_connect (menu_item, "activate",
+					  G_CALLBACK (zoom_in), info);
+	gtk_widget_show (menu_item);
+	gtk_menu_shell_append (GTK_MENU_SHELL (popup_menu), menu_item);
+	gtk_widget_add_accelerator(menu_item, "activate", info->accel_group, GDK_plus, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
+
+	menu_item = gtk_image_menu_item_new_from_stock (GTK_STOCK_ZOOM_OUT, NULL);
+	g_signal_connect (menu_item, "activate",
+					  G_CALLBACK (zoom_out), info);
+	gtk_widget_show (menu_item);
+	gtk_menu_shell_append (GTK_MENU_SHELL (popup_menu), menu_item);
+	gtk_widget_add_accelerator(menu_item, "activate", info->accel_group, GDK_minus, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
+
 	info->popup_menu = popup_menu;
 }
 
@@ -448,7 +543,7 @@ quit_cb (GtkWidget *win, gpointer unused)
 static void
 view_pixbuf (ViewerCbInfo * info, int xid, const char * color)
 {
-	GtkWidget *win, *img, *scroll;
+	GtkWidget *win, *scroll;
 	gint img_width, img_height, win_width, win_height;
 	GdkColor bg_color;
 
@@ -488,7 +583,7 @@ view_pixbuf (ViewerCbInfo * info, int xid, const char * color)
 	g_signal_connect(G_OBJECT(win), "delete_event", G_CALLBACK(quit_cb), NULL);	
 
 	/* create a new image */
-	img = gtk_image_new_from_pixbuf (info->pixbuf);
+	info->image = gtk_image_new_from_pixbuf (info->pixbuf);
 
 	/* pack the window with the image */
 	if ((xid > 0 && (img_width > win_width || img_height > win_height))
@@ -500,10 +595,10 @@ view_pixbuf (ViewerCbInfo * info, int xid, const char * color)
 		gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(scroll),
 										GTK_POLICY_AUTOMATIC,
 										GTK_POLICY_AUTOMATIC);
-		gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW(scroll), img);
+		gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW(scroll), info->image);
 		gtk_container_add(GTK_CONTAINER(win), scroll);
 	} else {
-		gtk_container_add(GTK_CONTAINER(win), img);
+		gtk_container_add(GTK_CONTAINER(win), info->image);
 		gtk_window_set_default_size(GTK_WINDOW(win), img_width, img_height);
 	}
 
@@ -511,7 +606,7 @@ view_pixbuf (ViewerCbInfo * info, int xid, const char * color)
 		{
 			if (gdk_color_parse (color, &bg_color))
 				{
-					GtkWidget * parent_widget = gtk_widget_get_parent(img);
+					GtkWidget * parent_widget = gtk_widget_get_parent(info->image);
 
 					if (gdk_colormap_alloc_color (gtk_widget_get_colormap(parent_widget), &bg_color, FALSE, TRUE))
 						gtk_widget_modify_bg (parent_widget, GTK_STATE_NORMAL, &bg_color);
@@ -623,7 +718,7 @@ main (int argc, char **argv)
 	gtk_init (&argc, &argv) ;
 	rsvg_init ();
 
-	rsvg_set_default_dpi_x_y (dpi_x, dpi_y);
+	rsvg_set_default_dpi (dpi_x, dpi_y);
 	
 	/* if both are unspecified, assume user wants to zoom the pixbuf in at least 1 dimension */
 	if (width == -1 && height == -1)
@@ -661,7 +756,7 @@ main (int argc, char **argv)
 
 		for (;;)
 			{
-				char buf[1024 * 8];
+				unsigned char buf[1024 * 8];
 				size_t nread = fread (buf, 1, sizeof(buf), stdin);
 			
 				if (nread > 0)
